import { Query } from '@kevisual/query';
import type { Result, DataOpts } from '@kevisual/query/query';
import { setBaseResponse } from '@kevisual/query/query';

export type SimpleObject = Record<string, any>;

export type SearchOpts = {
  page: number;
  pageSize: number;
  search?: string;
  sort?: string; // DESC, ASC
  markType?: string; // 类型
  [key: string]: any;
};

export type QueryMarkOpts<T extends SimpleObject = SimpleObject> = {
  query?: Query;
  isBrowser?: boolean;
  onLoad?: () => void;
} & T;

export type QueryMarkData = {
  id?: string;
  title?: string;
  description?: string;
  [key: string]: any;
};
export type QueryMarkResult = {
  accessToken: string;
  refreshToken: string;
};

export class QueryMarkBase<T extends SimpleObject = SimpleObject> {
  query: Query;
  isBrowser: boolean;
  load?: boolean;
  storage: Storage;
  onLoad?: () => void;

  constructor(opts?: QueryMarkOpts<T>) {
    this.query = opts?.query || new Query();
    this.isBrowser = opts?.isBrowser ?? true;
    this.init();
    this.onLoad = opts?.onLoad;
  }
  setQuery(query: Query) {
    this.query = query;
  }
  private async init() {
    this.load = true;
    this.onLoad?.();
  }

  async post<T = any>(data: any, opts?: DataOpts) {
    try {
      return this.query.post<T>({ path: 'mark', ...data }, opts);
    } catch (error) {
      console.log('error', error);
      return {
        code: 400,
      } as any;
    }
  }

  async getMarkList(search: SearchOpts, opts?: DataOpts) {
    return this.post({ key: 'list', ...search }, opts);
  }

  async getMark(id: string, opts?: DataOpts) {
    return this.post({ key: 'get', id }, opts);
  }

  async updateMark(id: string, data: any, opts?: DataOpts) {
    return this.post({ key: 'update', id, ...data }, opts);
  }
}
export class QueryMark extends QueryMarkBase<SimpleObject> {
  markType: string;
  constructor(opts?: QueryMarkOpts) {
    super(opts);
    this.markType = opts?.markType || 'simple';
  }
  async getMarkList(search: SearchOpts, opts?: DataOpts) {
    return this.post({ key: 'list', ...search, markType: this.markType }, opts);
  }
}
